<?php

/*
 * Session Management for PHP3
 *
 * Copyright (c) 1998-2000 NetUSE AG
 *                    Boris Erdmann, Kristian Koehntopp
 *
 * $Id: mysql.php,v 1.1.1.1 2008/10/13 06:13:37 winchild Exp $
 *
 */ 

class DB_Sql {
  
  /* public: connection parameters */
  var $Host     = "";
  var $Database = "";
  var $User     = "";
  var $Password = "";

  /* public: configuration parameters */
  var $Auto_Free     = 0;     ## Set to 1 for automatic mysql_free_result()
  var $Debug         = 0;     ## Set to 1 for debugging messages.
  var $Halt_On_Error = "yes"; ## "yes" (halt with message), "no" (ignore errors quietly), "report" (ignore errror, but spit a warning)
  var $PConnect      = 0;     ## Set to 1 to use persistent database connections
  var $Seq_Table     = "db_sequence";

  /* public: result array and current row number */
  var $Record   = array();
  var $Row;

  /* public: current error number and error text */
  var $Errno    = 0;
  var $Error    = "";

  /* public: this is an api revision, not a CVS revision. */
  var $type     = "mysql";
  var $revision = "1.2";

  /* private: link and query handles */
  var $Link_ID  = 0;
  var $Query_ID = 0;

  var $locked   = false;      ## set to true while we have a lock

  /* public: constructor */
  function DB_Sql($query = "") {
      $this->query($query);
  }

  /* public: some trivial reporting */
  function link_id() {
    return $this->Link_ID;
  }

  function query_id() {
    return $this->Query_ID;
  }

  /* public: connection management */
  function connect($Database = "", $Host = "", $User = "", $Password = "") {

	global $db_server, $db_user, $db_name, $db_pass;

    /* Handle defaults */
    if ("" == $Database) $Database = $db_name;
    if ("" == $Host) $Host = $db_server;
    if ("" == $User) $User = $db_user;
    if ("" == $Password) $Password = $db_pass;
      
    /* establish connection, select database */
    if ( 0 == $this->Link_ID ) {
    
      if(!$this->PConnect) {
        $this->Link_ID = mysql_connect($Host, $User, $Password);
      } else {
        $this->Link_ID = mysql_pconnect($Host, $User, $Password); 
      }
      if (!$this->Link_ID) {
        $this->halt("connect('$Host', '$User', '$Password') failed.");
        return 0;
      }

//echo "LINE=".__LINE__."($Host, $User, $Password, $Database)";
      if (!@mysql_select_db($Database,$this->Link_ID)) {
        $this->halt("cannot use database ".$Database);
        return 0;
      }
    }

    return $this->Link_ID;
  }

  /* public: close connection */
  function close() {
    $this->free();
    @mysql_close($this->Link_ID);
  }

  /* public: discard the query result */
  function free() {
      @mysql_free_result($this->Query_ID);
      $this->Query_ID = 0;
  }

  /* public: perform a query */
  function query($Query_String) {
    /* No empty queries, please, since PHP4 chokes on them. */
    if ($Query_String == "")
      /* The empty query string is passed on from the constructor,
       * when calling the class without a query, e.g. in situations
       * like these: '$db = new DB_Sql_Subclass;'
       */
      return 0;

    if (!$this->connect()) {
      return 0; /* we already complained in connect() about that. */
    };

    # New query, discard previous result.
    if ($this->Query_ID) {
      $this->free();
    }

    if ($this->Debug)
      printf("Debug: query = %s<br>\n", $Query_String);

    $this->Query_ID = @mysql_query($Query_String,$this->Link_ID);
    $this->Row   = 0;
    $this->Errno = mysql_errno();
    $this->Error = mysql_error();
//    if (!$this->Query_ID) {
//     $this->halt("Invalid SQL: ".$Query_String);
//    }

    # Will return nada if it fails. That's fine.
    return $this->Query_ID;
  }

  /* public: walk result set */
  function next_record() {
    if (!$this->Query_ID) {
      $this->halt("next_record called with no query pending.");
      return 0;
    }

    $this->Record = @mysql_fetch_array($this->Query_ID);
    $this->Row   += 1;
    $this->Errno  = mysql_errno();
    $this->Error  = mysql_error();

    $stat = is_array($this->Record);
    if (!$stat && $this->Auto_Free) {
      $this->free();
    }
    return $stat;
  }

  /* public: position in result set */
  function seek($pos = 0) {
    $status = @mysql_data_seek($this->Query_ID, $pos);
    if ($status)
      $this->Row = $pos;
    else {
      $this->halt("seek($pos) failed: result has ".$this->num_rows()." rows.");

      /* half assed attempt to save the day, 
       * but do not consider this documented or even
       * desireable behaviour.
       */
      @mysql_data_seek($this->Query_ID, $this->num_rows());
      $this->Row = $this->num_rows();
      return 0;
    }

    return 1;
  }

  /* public: table locking */
  function lock($table, $mode = "write") {
    $query = "lock tables ";
    if(is_array($table)) {
      while(list($key,$value) = each($table)) {
        // text keys are "read", "read local", "write", "low priority write"
        if(is_int($key)) $key = $mode;
        if(strpos($value, ",")) {
          $query .= str_replace(",", " $key, ", $value) . " $key, ";
        } else {
          $query .= "$value $key, ";
        }
      }
      $query = substr($query, 0, -2);
    } elseif(strpos($table, ",")) {
      $query .= str_replace(",", " $mode, ", $table) . " $mode";
    } else {
      $query .= "$table $mode";
    }
    if(!$this->query($query)) {
      $this->halt("lock() failed.");
      return false;
    }
    $this->locked = true;
    return true;
  }
  
  function unlock() {

    // set before unlock to avoid potential loop
    $this->locked = false;

    if(!$this->query("unlock tables")) {
      $this->halt("unlock() failed.");
      return false;
    }
    return true;
  }

  /* public: evaluate the result (size, width) */
  function affected_rows() {
    return @mysql_affected_rows($this->Link_ID);
  }

  function num_rows() {
    return @mysql_num_rows($this->Query_ID);
  }

  function num_fields() {
    return @mysql_num_fields($this->Query_ID);
  }

  /* public: shorthand notation */
  function nf() {
    return $this->num_rows();
  }

  function np() {
    print $this->num_rows();
  }

  function f($Name) {
    if (isset($this->Record[$Name])) {
      return $this->Record[$Name];
    }
  }

  function p($Name) {
    if (isset($this->Record[$Name])) {
      print $this->Record[$Name];
    }
  }

  /* public: sequence numbers */
  function nextid($seq_name) {
    /* if no current lock, lock sequence table */
    if(!$this->locked) {
      if($this->lock($this->Seq_Table)) {
        $locked = true;
      } else {
        $this->halt("cannot lock ".$this->Seq_Table." - has it been created?");
        return 0;
      }
    }
    
    /* get sequence number and increment */
    $q = sprintf("select nextid from %s where seq_name = '%s'",
               $this->Seq_Table,
               $seq_name);
    if(!$this->query($q)) {
      $this->halt('query failed in nextid: '.$q);
      return 0;
    }
    
    /* No current value, make one */
    if(!$this->next_record()) {
      $currentid = 0;
      $q = sprintf("insert into %s values('%s', %s)",
                 $this->Seq_Table,
                 $seq_name,
                 $currentid);
      if(!$this->query($q)) {
        $this->halt('query failed in nextid: '.$q);
        return 0;
      }
    } else {
      $currentid = $this->f("nextid");
    }
    $nextid = $currentid + 1;
    $q = sprintf("update %s set nextid = '%s' where seq_name = '%s'",
               $this->Seq_Table,
               $nextid,
               $seq_name);
    if(!$this->query($q)) {
      $this->halt('query failed in nextid: '.$q);
      return 0;
    }
    
    /* if nextid() locked the sequence table, unlock it */
    if($locked) {
      $this->unlock();
    }
    
    return $nextid;
  }

  /* public: return table metadata */
  function metadata($table = "", $full = false) {
    $count = 0;
    $id    = 0;
    $res   = array();

    /*
     * Due to compatibility problems with Table we changed the behavior
     * of metadata();
     * depending on $full, metadata returns the following values:
     *
     * - full is false (default):
     * $result[]:
     *   [0]["table"]  table name
     *   [0]["name"]   field name
     *   [0]["type"]   field type
     *   [0]["len"]    field length
     *   [0]["flags"]  field flags
     *
     * - full is true
     * $result[]:
     *   ["num_fields"] number of metadata records
     *   [0]["table"]  table name
     *   [0]["name"]   field name
     *   [0]["type"]   field type
     *   [0]["len"]    field length
     *   [0]["flags"]  field flags
     *   ["meta"][field name]  index of field named "field name"
     *   This last one could be used if you have a field name, but no index.
     *   Test:  if (isset($result['meta']['myfield'])) { ...
     */

    // if no $table specified, assume that we are working with a query
    // result
    if ($table) {
      $this->connect();
      $id = @mysql_list_fields($this->Database, $table);
      if (!$id) {
        $this->halt("Metadata query failed.");
        return false;
      }
    } else {
      $id = $this->Query_ID; 
      if (!$id) {
        $this->halt("No query specified.");
        return false;
      }
    }
 
    $count = @mysql_num_fields($id);

    // made this IF due to performance (one if is faster than $count if's)
    if (!$full) {
      for ($i=0; $i<$count; $i++) {
        $res[$i]["table"] = @mysql_field_table ($id, $i);
        $res[$i]["name"]  = @mysql_field_name  ($id, $i);
        $res[$i]["type"]  = @mysql_field_type  ($id, $i);
        $res[$i]["len"]   = @mysql_field_len   ($id, $i);
        $res[$i]["flags"] = @mysql_field_flags ($id, $i);
      }
    } else { // full
      $res["num_fields"]= $count;
    
      for ($i=0; $i<$count; $i++) {
        $res[$i]["table"] = @mysql_field_table ($id, $i);
        $res[$i]["name"]  = @mysql_field_name  ($id, $i);
        $res[$i]["type"]  = @mysql_field_type  ($id, $i);
        $res[$i]["len"]   = @mysql_field_len   ($id, $i);
        $res[$i]["flags"] = @mysql_field_flags ($id, $i);
        $res["meta"][$res[$i]["name"]] = $i;
      }
    }
    
    // free the result only if we were called on a table
    if ($table) {
      @mysql_free_result($id);
    }
    return $res;
  }

  /* public: find available table names */
  function table_names() {
    $this->connect();
    $h = @mysql_query("show tables", $this->Link_ID);
    $i = 0;
    while ($info = @mysql_fetch_row($h)) {
      $return[$i]["table_name"]      = $info[0];
      $return[$i]["tablespace_name"] = $this->Database;
      $return[$i]["database"]        = $this->Database;
      $i++;
    }
    
    @mysql_free_result($h);
    return $return;
  }

  /* private: error handling */
  function halt($msg) {
    $this->Error = @mysql_error($this->Link_ID);
    $this->Errno = @mysql_errno($this->Link_ID);

    if ($this->locked) {
      $this->unlock();
    }

    if ($this->Halt_On_Error == "no")
      return;

    $this->haltmsg($msg);

    if ($this->Halt_On_Error != "report")
      die("Session halted.");
  }

  function haltmsg($msg) {
    printf("</td></tr></table><b>Database error:</b> %s<br>\n", $msg);
    printf("<b>MySQL Error</b>: %s (%s)<br>\n",
      $this->Errno,
      $this->Error);
  }

  function insert_id () {
        return mysql_insert_id ($this->Link_ID);
  }

}

// 발송메일 MAILJOB 테이블에 등록.
//
//	$email = array();
//
function mailsend ($email, $subject, $body)
{
	global $mail_mysql_database, $mail_mysql_host, $mail_mysql_user, $mail_mysql_pass;

	// MAILJOB DATABASE
	$db = new DB_Sql();
//	$db->connect ($mail_mysql_database, $mail_mysql_host, $mail_mysql_user, $mail_mysql_pass);

	$v_subject = addslashes($subject);	// 제목
	$v_body = addslashes($body);	// 내용

	// JOB 테이블에 등록, 데몬이 먼저 긁어갈수 있으므로 상태는 'X' 테이블 완료후에 INSERT.
	$query = "INSERT INTO mailjob.mailjob (rdate,status,email,cat_code,memo,id,ts) ".
		"VALUES (NOW(),'I','direct@sds.co.kr','H','$v_subject','".$_SERVER["PHP_AUTH_USER"]."',NOW())";
//echo "QUERY=(".__LINE__.")=[$query]<br>\n";

	if(!$db->query($query)) {
		PutLog (__FILE__,__LINE__,ERROR,$query,TRUE);
		$db->close();
		exit;
	}
  	$job_sid = $db->insert_id ();	// 마지막으로 집어넣은 INSERT ID 가져오기.

	while (($to_email = array_pop($email)) != NULL)
	{
		$mail_header = "From: SDS INTRANET<".REPLYADDRESS.">\n";
		$mail_header .= "To: ".$to_email." <".$to_email.">\n";
//		$mail_header .= "Subject: $subject\n";
		// UTF-8 로 제목 인코딩.
		$mail_header .= "Subject: =?utf-8?B?".base64_encode($v_subject)."?=\n";
		$mail_header .= "MIME-Version: 1.0\n";   
		$mail_header .= "Content-Type: text/html; charset=UTF-8\n"; // Mime type     
		$mail_header .= "Content-Transfer-Encoding: 8BIT";
		$mail_header = addslashes ($mail_header);

		$query = "INSERT INTO mailjob.mailsend (job_sid,rdate,email_status,email,header,body,id,ts,subject) ".
			"VALUES ('".$job_sid."',NOW(),'W','".$to_email."','".$mail_header."','".$v_body."',".
			"'".$_SERVER["PHP_AUTH_USER"]."',NOW(),'$subject')";
//echo "QUERY=(".__LINE__.")=[$query]<br>\n";
		if (!$db->query ($query)) {
			PutLog (__FILE__,__LINE__,ERROR,$query.":".$db->Error,TRUE);
			$db->close();
			exit;
		}
	}

	// 메일발송 데몬이 긁어 가도록 'W' 로 상태 변경.
	$query = "UPDATE mailjob.mailjob SET status='W' WHERE sid='".$job_sid."'";
//echo "QUERY(".__LINE__.")=$query<br>\n";
	if(!$db->query($query)) {
		PutLog (__FILE__,__LINE__,ERROR,$query.":".$db->Error,TRUE);
		$db->close();
		exit;
	}
}

//====================================================
//	부서팀장에게 메일발송
//====================================================
function mail_to_post ($db, $work_id, $from_post, $subject, $body, $tag = "업무요청")
{
	// 해당부서 팀장 정보
	$query = "SELECT a.email,a.name FROM t_user a, t_post b ".
		"WHERE b.post_uid='$from_post' AND b.manager=a.emp_no";
	if (!$db->query($query))
	{
		PutLog (__FILE__,__LINE__,ERROR,$query.":".$db->Error,TRUE);
		exit;
	}
	$cnt =  $db->nf();
	if ($cnt == 0)
	{
		PutLog (__FILE__,__LINE__,ERROR,$query.": Empty!",TRUE);
		return;
	}
	$email = array();
	while ($db->next_record()) array_push ($email, $db->f("email"));

	mailsend ($email, $subject, "\n\n인트라넷의 \"$tag\"에서 자동으로 발송된 메일 입니다.<br>\n".
		"\n-------------------------------------------------------<br>\n" . $body);

}

//====================================================
//	부터 팀원전체에게 메일발송
//====================================================
function mail_to_postmembers ($db, $work_id, $from_post, $subject, $body, $tag)
{
	// 해당부서 팀장 정보
	$query = "SELECT email FROM t_user WHERE post='$from_post' AND kt_disable=0";
	if (!$db->query($query))
	{
		PutLog (__FILE__,__LINE__,ERROR,$query.":".$db->Error,TRUE);
		exit;
	}
	$cnt =  $db->nf();
	if ($cnt == 0)
	{
		PutLog (__FILE__,__LINE__,ERROR,$query.": Empty!",TRUE);
		return;
	}
	$email = array();
	while ($db->next_record()) array_push ($email, $db->f("email"));

	mailsend ($email, $subject, "\n\n인트라넷의 \"$tag\"에서 자동으로 발송된 메일 입니다.<br>\n".
		"\n-------------------------------------------------------<br>\n" . $body);

}

//====================================================
//	담당자에게 메일발송
//====================================================
function mail_to_user ($db, $work_id, $id, $subject, $body, $tag="업무요청")
{
	// 담당자 이메일
	$query = "SELECT email,name FROM t_user WHERE emp_no='$id'";
	if (!$db->query($query))
	{
		PutLog (__FILE__,__LINE__,ERROR,$query.":".$db->Error,TRUE);
		exit;
	}
	$cnt =  $db->nf();
	if ($cnt == 0)
	{
		PutLog (__FILE__,__LINE__,ERROR, $query.": Empty!",TRUE);
		return;
	}
	if ($db->next_record())
	{
		$email = array();
		array_push ($email, $db->f("email"));
		mailsend ($email, $subject, "\n\n인트라넷의 \"$tag\"에서 자동으로 발송된 메일 입니다.<br>\n".
			"\n-------------------------------------------------------<br>\n" . $body);
	}

}

//	출근데이터 여부를 체크하고
//	없으면 입력.
//
// mysql> desc kt_iots;
// +------------+-------------------------------------------+------+-----+---------------------+----------------+
// | Field      | Type                                      | Null | Key | Default             | Extra          |
// +------------+-------------------------------------------+------+-----+---------------------+----------------+
// | sid        | int(11)                                   | NO   | PRI | NULL                | auto_increment |
// | emp_no     | varchar(6)                                | NO   | MUL |                     |                |
// | tg_date    | date                                      | NO   |     | 0000-00-00          |                |
// | start_ts   | time                                      | NO   | MUL | 00:00:00            |                |
// | end_ts     | time                                      | NO   | MUL | 00:00:00            |                |
// | memo       | tinytext                                  | YES  |     | NULL                |                |
// | prov_flag  | enum('Y','N')                             | YES  |     | NULL                |                |
// | wkid       | varchar(6)                                | NO   |     |                     |                |
// | ts         | datetime                                  | NO   |     | 0000-00-00 00:00:00 |                |
// | status     | enum('N','A','L','H','B','S','P','Y','R') | NO   |     | N                   |                |
// | yrts       | int(11)                                   | NO   |     | 0                   |                |
// | ip         | varchar(16)                               | YES  |     | NULL                |                |
// | check_flag | enum('Y','N')                             | YES  |     | N                   |                |
// | mailsend   | enum('Y','N')                             | YES  |     | N                   |                |
// +------------+-------------------------------------------+------+-----+---------------------+----------------+
// 14 rows in set (0.01 sec)
//
function attend_insert ($mysql, $emp_no)
{
	$rc = 0;

	// 오늘 입력된 데이터가 있는지 확인, 새벽5시 이후의 것을
	// 해당 일자의 출근 // 데이터로 인정.

	$query = "SELECT check_flag,status FROM kt_iots WHERE tg_date = NOW() AND emp_no='$emp_no'";
//PutLog (__FILE__,__LINE__,DEBUG,$query,TRUE);
	if (!$mysql->query($query))
	{
		PutLog (__FILE__,__LINE__,ERROR,$query.":".$mysql->Error,FALSE);
		$mysql->close();
		exit;
	}
	if ($mysql->nf() > 0 && $mysql->next_record())
	{
			// 문자셋을 리턴하므로 숫자값을 리턴하더라도 문자로 리턴해야 한다.
		if ($mysql->f("check_flag") == 'Y') return "0";
		return $mysql->f("status");
	}


	// 없으면 입력.

	// 근태에 해당되는 직원의 출근시간 FETCH.
	$query = "SELECT st_tm FROM t_user WHERE emp_no='$emp_no'";
//PutLog (__FILE__,__LINE__,DEBUG,$query,TRUE);
	if (!$mysql->query($query))
	{
		PutLog (__FILE__,__LINE__,ERROR,$query.":".$mysql->Error,FALSE);
		$mysql->close();
		exit;
	}
	if ($mysql->nf() == 0)
	{
		// 근태 출근시간 기록되어 있지 않음.
		PutLog (__FILE__,__LINE__,ERROR,$query.":".$mysql->Error,FALSE);
		$mysql->close();
		exit;
	}

	if ($mysql->next_record() == 0)
	{
		PutLog (__FILE__,__LINE__,ERROR,$query.":".$mysql->Error,FALSE);
		$mysql->close();
		exit;
	}

	$memo = "";

	// 요일체크
	$wday = getdate ();
	if ($wday["wday"] == 0 || $wday["wday"] == 6)
	{
		// 주일이나 토요일
		$status = KT_REST;	// 휴일근무
		$memo = "휴일근무";
	}
	else
	{
		// 휴일근무인가 체크
		$query = "SELECT 1 FROM kt_holiday WHERE tdate=NOW()";
//PutLog (__FILE__,__LINE__,ERROR,$query,TRUE);

		if(!$mysql->query($query))
		{
			PutLog (__FILE__,__LINE__,ERROR,$query.":".$mysql->Error,FALSE);
			$mysql->close();
			exit;
		}
		if ($mysql->nf() > 0)
		{
			$status = KT_REST;	// 휴일근무
			$memo = "휴일근무";
		}
		else
		{
			// 휴일근무가 아닌경우에 만 지각체크
			$now_ms = date("H:i");
			$reg_ms = $mysql->f("st_tm");

			if ($reg_ms > $now_ms)
			{
				$status = KT_NORMAL;	// 정상
				$diff_ms = 0;
			}
			else
			{
				if ($reg_ms > ($now_ms + (OVER_ALLOW_TIME/60) + 1))
				{
					$status = KT_LATENESS;	// 지각
					$diff_ms = 0;	// 연차에서는 감산하지 않는다.
				}
				else
				{
					// 5분이상 늦은 경우에만 연차에서 감산한다.
					$status = KT_LATENESS;	// 지각
					$diff_ms = substr ($reg_ms,0,2) * 60 + substr ($reg_ms,3,2);
					$diff_ms -= substr ($now_ms,0,2) * 60 + substr ($now_ms,3,2);
				}
			}
		}
	}

	// 출근기록 추가.
	$query = "INSERT INTO kt_iots
		(emp_no, tg_date, start_ts, status, yrts, memo, ip, wkid, ts) VALUES (
		'$emp_no', NOW(), '$now_ms','$status','$diff_ms','$memo','".$_SERVER["REMOTE_ADDR"]."','ATTEND',NOW())";
//PutLog (__FILE__,__LINE__,DEBUG,$query,TRUE);
	if (!$mysql->query ($query))
	{
		PutLog (__FILE__,__LINE__,ERROR,$query.":".$mysql->Error,FALSE);
		$mysql->close();
		exit;
	}
  	$sid = $mysql->insert_id ();

	return $status;
}

?>
